@@ -28,6 +28,7 @@
 #include <gioerror.h>
 #ifdef G_OS_WIN32
 #include <windows.h>
+#include <io.h>
 #endif
 #include "gcancellable.h"
 #include "glibintl.h"
@@ -326,11 +327,11 @@ g_cancellable_reset (GCancellable *cancellable)
 
   G_LOCK(cancellable);
   
-  if (cancellable->cancelled_running)
+  while (cancellable->cancelled_running)
     {
-      g_critical ("Can't reset a cancellable during an active operation");
-      G_UNLOCK(cancellable);
-      return;
+      cancellable->cancelled_running_waiting = TRUE;
+      g_cond_wait (cancellable_cond,
+                   g_static_mutex_get_mutex (& G_LOCK_NAME (cancellable)));
     }
   
   if (cancellable->cancelled)
