@@ -111,7 +111,7 @@ dup_noninherited (int fd,
 		   GetCurrentProcess (), &filehandle,
 		   0, FALSE, DUPLICATE_SAME_ACCESS);
   close (fd);
-  return _open_osfhandle ((gssize) filehandle, mode | _O_NOINHERIT);
+  return _open_osfhandle ((gintptr) filehandle, mode | _O_NOINHERIT);
 }
 
 #ifndef GSPAWN_HELPER
@@ -308,22 +308,22 @@ make_pipe (gint     p[2],
  */
 static gboolean
 read_helper_report (int      fd,
-		    gssize   report[2],
+		    gintptr  report[2],
 		    GError **error)
 {
   gint bytes = 0;
   
-  while (bytes < sizeof(gssize)*2)
+  while (bytes < sizeof(gintptr)*2)
     {
       gint chunk;
 
       if (debug)
 	g_print ("%s:read_helper_report: read %" G_GSIZE_FORMAT "...\n",
 		 __FILE__,
-		 sizeof(gssize)*2 - bytes);
+		 sizeof(gintptr)*2 - bytes);
 
       chunk = read (fd, ((gchar*)report) + bytes,
-		    sizeof(gssize)*2 - bytes);
+		    sizeof(gintptr)*2 - bytes);
 
       if (debug)
 	g_print ("...got %d bytes\n", chunk);
@@ -349,14 +349,14 @@ read_helper_report (int      fd,
 	bytes += chunk;
     }
 
-  if (bytes < sizeof(gssize)*2)
+  if (bytes < sizeof(gintptr)*2)
     return FALSE;
 
   return TRUE;
 }
 
 static void
-set_child_error (gssize       report[2],
+set_child_error (gintptr      report[2],
 		 const gchar *working_directory,
 		 GError     **error)
 {
@@ -422,14 +422,12 @@ do_spawn_directly (gint                 *exit_status,
 		   gchar               **argv,
 		   char                **envp,
 		   char                **protected_argv,
-		   GSpawnChildSetupFunc  child_setup,
-		   gpointer              user_data,
 		   GPid                 *child_handle,
 		   GError              **error)     
 {
   const int mode = (exit_status == NULL) ? P_NOWAIT : P_WAIT;
   char **new_argv;
-  gssize rc = -1;
+  gintptr rc = -1;
   int saved_errno;
   GError *conv_error = NULL;
   gint conv_error_index;
@@ -471,9 +469,6 @@ do_spawn_directly (gint                 *exit_status,
       return FALSE;
     }
 
-  if (child_setup)
-    (* child_setup) (user_data);
-
   if (flags & G_SPAWN_SEARCH_PATH)
     if (wenvp != NULL)
       rc = _wspawnvpe (mode, wargv0, (const wchar_t **) wargv, (const wchar_t **) wenvp);
@@ -524,7 +519,6 @@ do_spawn_with_pipes (gint                 *exit_status,
 		     char                **envp,
 		     GSpawnFlags           flags,
 		     GSpawnChildSetupFunc  child_setup,
-		     gpointer              user_data,
 		     GPid                 *child_handle,
 		     gint                 *standard_input,
 		     gint                 *standard_output,
@@ -536,7 +530,7 @@ do_spawn_with_pipes (gint                 *exit_status,
   char args[ARG_COUNT][10];
   char **new_argv;
   int i;
-  gssize rc = -1;
+  gintptr rc = -1;
   int saved_errno;
   int argc;
   int stdin_pipe[2] = { -1, -1 };
@@ -544,20 +538,20 @@ do_spawn_with_pipes (gint                 *exit_status,
   int stderr_pipe[2] = { -1, -1 };
   int child_err_report_pipe[2] = { -1, -1 };
   int helper_sync_pipe[2] = { -1, -1 };
-  gssize helper_report[2];
+  gintptr helper_report[2];
   static gboolean warned_about_child_setup = FALSE;
   GError *conv_error = NULL;
   gint conv_error_index;
   gchar *helper_process;
   CONSOLE_CURSOR_INFO cursor_info;
   wchar_t *whelper, **wargv, **wenvp;
-  extern gchar *_glib_get_installation_directory (void);
-  gchar *glib_top;
+  extern gchar *_glib_get_dll_directory (void);
+  gchar *glib_dll_directory;
 
   if (child_setup && !warned_about_child_setup)
     {
       warned_about_child_setup = TRUE;
-      g_warning ("passing a child setup function to the g_spawn functions is pointless and dangerous on Win32");
+      g_warning ("passing a child setup function to the g_spawn functions is pointless on Windows and it is ignored");
     }
 
   argc = protect_argv (argv, &protected_argv);
@@ -573,8 +567,7 @@ do_spawn_with_pipes (gint                 *exit_status,
       gboolean retval =
 	do_spawn_directly (exit_status, do_return_handle, flags,
 			   argv, envp, protected_argv,
-			   child_setup, user_data, child_handle,
-			   error);
+			   child_handle, error);
       g_strfreev (protected_argv);
       return retval;
     }
@@ -600,11 +593,11 @@ do_spawn_with_pipes (gint                 *exit_status,
   else
     helper_process = HELPER_PROCESS ".exe";
   
-  glib_top = _glib_get_installation_directory ();
-  if (glib_top != NULL)
+  glib_dll_directory = _glib_get_dll_directory ();
+  if (glib_dll_directory != NULL)
     {
-      helper_process = g_build_filename (glib_top, "bin", helper_process, NULL);
-      g_free (glib_top);
+      helper_process = g_build_filename (glib_dll_directory, helper_process, NULL);
+      g_free (glib_dll_directory);
     }
   else
     helper_process = g_strdup (helper_process);
@@ -752,9 +745,6 @@ do_spawn_with_pipes (gint                 *exit_status,
       goto cleanup_and_fail;
     }
 
-  if (child_setup)
-    (* child_setup) (user_data);
-
   whelper = g_utf8_to_utf16 (helper_process, -1, NULL, NULL, NULL);
   g_free (helper_process);
 
@@ -936,7 +926,6 @@ g_spawn_sync_utf8 (const gchar          *working_directory,
 			    envp,
 			    flags,
 			    child_setup,
-			    user_data,
 			    NULL,
 			    NULL,
 			    standard_output ? &outpipe : NULL,
@@ -1074,7 +1063,7 @@ g_spawn_sync_utf8 (const gchar          *working_directory,
       /* Helper process was involved. Read its report now after the
        * grandchild has finished.
        */
-      gssize helper_report[2];
+      gintptr helper_report[2];
 
       if (!read_helper_report (reportpipe, helper_report, error))
 	failed = TRUE;
@@ -1157,7 +1146,6 @@ g_spawn_async_with_pipes_utf8 (const gchar          *working_directory,
 			      envp,
 			      flags,
 			      child_setup,
-			      user_data,
 			      child_handle,
 			      standard_input,
 			      standard_output,
