@@ -880,6 +880,7 @@ write_all (gint fd, gconstpointer vbuf, gsize to_write)
   return TRUE;
 }
 
+G_GNUC_NORETURN
 static void
 write_err_and_exit (gint fd, gint msg)
 {
@@ -1461,11 +1462,12 @@ make_pipe (gint     p[2],
 {
   if (pipe (p) < 0)
     {
+      gint errsv = errno;
       g_set_error (error,
                    G_SPAWN_ERROR,
                    G_SPAWN_ERROR_FAILED,
                    _("Failed to create pipe for communicating with child process (%s)"),
-                   g_strerror (errno));
+                   g_strerror (errsv));
       return FALSE;
     }
   else
